//---------------------------
// Entetes exif d'après :
//   Version:   $Rev: 1039 $
//   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
//   History:   10-Dec-03, ahu: created
//---------------------------
// Inclusions exif
//#include "../exiflib/rcsid.hpp"
//EXIV2_RCSID("@(#) $Id: exiv2.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
//#if defined _MSC_VER || __MINGW32__
//  #include "../exiflib/exv_msvc.h"
//#else
//  #include "../exiflib/exv_conf.h"
//#endif
/*
#include "exv_conf.h"

#include "exiv2.hpp"
#include "actions.hpp"
#include "utils.hpp"
#include "i18n.h"      // NLS support.
*/

#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <cstring>
#include <cassert>
#include <exiv2.hpp>
//---------------------------
#include "apn.h"

QStringList goodTags;

//---------------------------
void init_APN_Tags()
{
  goodTags << "Image.Make" << "Image.Model"  << "Image.Software"
           << "OwnerName"  << "SerialNumber" << "FirmwareVersion";
}

int allExifsToModel(QString filename, QStandardItemModel &model)
{
  model.clear();
  int cpt=0;
  try {
    const char *ficname = qPrintable(filename);
    Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(ficname);
    assert(image.get() != 0);
    image->readMetadata();

    Exiv2::ExifData &exifData = image->exifData();
    if (exifData.empty()) {
      QList<QStandardItem *> mots;
      mots.append(new QStandardItem(QString(QT_TR_NOOP("Aucune information EXIF disponible"))));
      model.appendRow(mots);
      ++cpt;
      return cpt;
    }
    QStringList entete;
    entete << "Nom" << "Type" << "Compte" << "Tag" << "Valeur";
    model.setHorizontalHeaderLabels(entete);
    Exiv2::ExifData::const_iterator end = exifData.end();
    for (Exiv2::ExifData::const_iterator i = exifData.begin(); i != end; ++i) {
      QString tip = i->typeName();
//       if ((ui.ckAscii->isChecked())     && (tip != "Ascii"))        continue;
      QList<QStandardItem *> mots;
      mots.append(new QStandardItem(QString(i->key().c_str()).replace("Exif.","")));
      mots.append(new QStandardItem(tip));
      mots.append(new QStandardItem(QString("%1").arg(i->count())));
      mots.append(new QStandardItem(QString().sprintf("0x%04x",i->tag())));
      mots.append(new QStandardItem(QString(i->value().toString().c_str())));
      for(int i=0; i<mots.size(); ++i) mots.at(i)->setEditable(false);
      model.appendRow(mots);
      ++cpt;
    }
  }
  catch(const Exiv2::AnyError &error){
    QList<QStandardItem *> mots;
    mots.append(new QStandardItem(QString(QT_TR_NOOP("Erreur de lecture des données EXIF : %1")).arg(error.what())));
    model.appendRow(mots);
    ++cpt;
  }
  return cpt;
}

//---------------------------
Exifs::Exifs(QString filename)
{
  nomfic = filename;
  vide = true;
  cpt  = 1;
  try {
    const char *ficname = qPrintable(nomfic);
    Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(ficname);
    assert(image.get() != 0);
    image->readMetadata();

    Exiv2::ExifData &exifData = image->exifData();
    if (exifData.empty()) return;
    Exiv2::ExifData::const_iterator end = exifData.end();
    for (Exiv2::ExifData::const_iterator i = exifData.begin(); i != end; ++i) {
      QString tag = i->key().c_str();
      tag.replace("Exif.","");
      if (tag == QString("Photo.DateTimeOriginal"))
        dat = QDateTime::fromString(i->value().toString().c_str(), "yyyy:MM:dd hh:mm:ss");
      for(int n=0; n<goodTags.size(); ++n)
        if (tag.contains(goodTags.at(n), Qt::CaseInsensitive))
          profil[tag] = i->value().toString().c_str();
    }
    vide = profil.isEmpty();
  }
  catch(const Exiv2::AnyError &error){
    erreur = error.what();
  }
}

void Exifs::fromSettings(QSettings &settings)
{
  QStringList cles = settings.allKeys();
  for(int i=0; i<cles.size();++i)
    profil[cles.at(i)] = settings.value(cles.at(i), "").toString();
  vide = (cles.size()==0);
}

void Exifs::save(QSettings &settings)
{
  settings.beginGroup("Exifs");
  QMapIterator<QString, QString> i(profil);
  while (i.hasNext()) {
    i.next();
    settings.setValue(i.key(), i.value());
  }
  settings.endGroup();
}

bool Exifs::operator==(const Exifs &other) const
{ // un même apn a les mêmes clés contenant les mêmes valeurs
  bool ok = true;
  QMapIterator<QString, QString> i(profil);
  while (i.hasNext()) {
    i.next();
    if (!other.profil.contains(i.key()))        // teste l'existence de la clé
     ok = false;
    else if (i.value()!=other.profil[i.key()])  // teste la valeur dans le tableau
     ok = false;
  }
  return ok;
}

void Exifs::profil2Model(QStandardItemModel &model)
{
  QMapIterator<QString, QString> i(profil);
  while (i.hasNext()) {
    i.next();
    QList<QStandardItem *> mots;
    mots.append(new QStandardItem(i.key()));
    mots.append(new QStandardItem(i.value()));
    mots.at(0)->setEditable(false);
    mots.at(1)->setEditable(false);
    model.appendRow(mots);
  }
}

//---------------------------

APN::APN(Exifs &ex) : profil(ex)
{
  nom       = "";
  suffixe   = "";
  tdays     = 0;
  tdec      = QTime();
  operation = 1;
}

APN::APN(QString name, QSettings &settings) : profil()
{
  settings.beginGroup(name);
  nom       = name;
  suffixe   = settings.value("suffixe", "").toString();
  tdays     = settings.value("tdays", 0 ).toInt();
  tdec      = settings.value("tdec", QTime() ).toTime();
  operation = settings.value("operation", 1 ).toInt();
    settings.beginGroup("Exifs");
    profil.fromSettings(settings);
    settings.endGroup();
  settings.endGroup();
}

void APN::save(QSettings &settings)
{
  settings.beginGroup(nom);
  settings.setValue("suffixe",   suffixe);
  settings.setValue("tdays",     tdays);
  settings.setValue("tdec",      tdec);
  settings.setValue("operation", operation);
  profil.save(settings);
  settings.endGroup();
}
