#ifndef APN_H
#define APN_H

#include <QtCore>
#include <QApplication>
#include <QStandardItemModel>

void init_APN_Tags();

///---------------------------
//! Rempli un modele depuis les données exif d'une image
int  allExifsToModel(QString filename, QStandardItemModel &model);

///---------------------------
class Exifs
{
  public :
    Exifs(QString filename);                 // depuis une image
    Exifs()                      { vide = true;   }
    bool isEmpty()               { return vide;   }
    bool isError()               { return erreur.isNull(); }
    QString fname() const        { return nomfic; }
    QString errMessage() const   { return erreur; }
    void add(int n = 1)          { cpt += n;      }
    int  count() const           { return cpt;    }
    QDateTime exifDateOriginal() { return dat;    }
    bool operator==(const Exifs &) const;    // compare les données exif
    void fromSettings(QSettings &settings);  // depuis les settings
    void profil2Model(QStandardItemModel &);
    void save(QSettings &settings);
  private :
    QMap<QString, QString>  profil;          // clés exif d'une image
    bool                    vide;            // Ya pas de données exif
    QString                 erreur;          // contient le texte de l'erreur de lecture des exifs
    QString                 nomfic;          // nom du fichier
    int                     cpt;             // compteur d'utilisation
    QDateTime               dat;             // date et heure de l'original
};

///---------------------------
class APN
{
  public :
    APN(Exifs &ex);                          // depuis des données exifs
    APN(QString name, QSettings &settings);  // depuis les settings
    void setName(const QString &n)          { nom = n;          }
    void setSuf(const QString &suf)         { suffixe = suf;    }
    void setDays(int d)                     { tdays = d;        }
    void setTime(const QTime &tim)          { tdec  = tim;      }
    void setOp(int op)                      { operation = op;   }
    QString Name() const                    { return nom;       }
    QString Suf()  const                    { return suffixe;   }
    int   Days()   const                    { return tdays;     }
    QTime Time()   const                    { return tdec;      }
    int   Op()     const                    { return operation; }
    Exifs   exif() const                    { return profil;    }
    bool isEqual(const Exifs &other) const  { return (profil==other); }  // compare avec des données exif
    void save(QSettings &settings);
  private :
    QString                 nom;             // nom du profil
    Exifs                   profil;          // clés exif à comparer
    QString                 suffixe;         // suffixe à rajouter
    int                     tdays;           // décalage en jours
    QTime                   tdec;            // décalage en hhmmss
    int                     operation;       // 0 keep time, 1 add time, 2 subtract time
};

#endif
