#include "dials.h"

extern int           hauteurLigne;
extern QList<APN>    apns;

//---------------------------
DialAPNList::DialAPNList(QWidget *parent, QString &rep, QStringList &images) :
  QDialog(parent)
{
  ui.setupUi(this);
  setModal(true);
  setWindowTitle(tr("New cameras"));
  ui.lbDir->setText(rep);
  initExs(rep, images);
  // liste des nouveaux apns
  namodel.clear();
  for(int l=0; l<exs.size(); ++l) {
    QList<QStandardItem *> mots;
    mots.append(new QStandardItem(newapns.at(l).Name()));
    mots.append(new QStandardItem(QString(tr("%1 pictures")).arg(exs.at(l).count())));
    mots.at(0)->setEditable(false);
    mots.at(1)->setEditable(false);
    namodel.appendRow(mots);
  }
  ui.vueExifs->setModel(&namodel);
  ui.vueExifs->verticalHeader()->hide();
  ui.vueExifs->horizontalHeader()->hide();
  for(int i=0; i<namodel.rowCount(); ++i) ui.vueExifs->setRowHeight(i,hauteurLigne);

  // liste des apns connus
  amodel.clear();
  for(int l=0; l<apns.size(); ++l) {
    QList<QStandardItem *> mots;
    mots.append(new QStandardItem(apns.at(l).Name()));
    mots.at(0)->setEditable(false);
    amodel.appendRow(mots);
  }
  ui.vueApns->setModel(&amodel);
  ui.vueApns->verticalHeader()->hide();
  ui.vueApns->horizontalHeader()->hide();
  for(int i=0; i<amodel.rowCount(); ++i) ui.vueApns->setRowHeight(i,hauteurLigne);

  // données exif trouvées
  ui.vueExifdata->setModel(&nemodel);
  ui.vueExifdata->verticalHeader()->hide();

  QObject::connect(ui.vueExifs->selectionModel(),  SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)),
                   this,       SLOT(changeExif(const QModelIndex &, const QModelIndex &)));
  ui.vueExifs->selectRow(0);

  // données exif des apns
  ui.vueApndata->setModel(&emodel);
  ui.vueApndata->verticalHeader()->hide();

  QObject::connect(ui.vueApns->selectionModel(),  SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)),
                   this,       SLOT(changeApn(const QModelIndex &, const QModelIndex &)));
  ui.vueApns->selectRow(0);
  QObject::connect(ui.btAdd,    SIGNAL(clicked()),       this, SLOT(ajoute()));
  QObject::connect(ui.btDelete, SIGNAL(clicked()),       this, SLOT(supprime()));
  QObject::connect(ui.btEdit,   SIGNAL(clicked()),       this, SLOT(edite()));
  QObject::connect(ui.vueExifs, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(ajoute()));
  QObject::connect(ui.vueApns,  SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(edite()));
}

void DialAPNList::initExs(QString &rep, QStringList &images)
{
  int cpt = 1;
  newapns.clear();
  for (int l=0; l<images.size(); ++l) {
    Exifs eximage(QString("%1/%2").arg(rep).arg(images[l]));
    if (eximage.isEmpty()) continue;
    int pos;
    if ((pos = exs.indexOf(eximage,0))!=-1) {
      pos = exs.indexOf(eximage,0);
      exs[pos].add();
    }
    else {
      exs.append(eximage);
      int mem = -1;
      for(int i=0; i<apns.size(); ++i)   // Cherche un apn déjà existant
        if (eximage==apns.at(i).exif())
          mem = i;
      if (mem==-1) {
        APN apn(eximage);
        apn.setName(QString(tr("Camera-%1")).arg(cpt));
        newapns.append(apn);
        ++cpt;
      }
      else {
        newapns.append(apns.at(mem));
      }
    }
  }
}

void DialAPNList::changeExif(const QModelIndex &index, const QModelIndex &)
{  //! affiche les données exif du nouvel apn sélectionné
  Exifs ex = exs.at(index.row());
  nemodel.clear();
  QStringList entete;
  entete << tr("Exif") << tr("Value");
  nemodel.setHorizontalHeaderLabels(entete);
  ex.profil2Model(nemodel);
  for(int i=0; i<nemodel.rowCount(); ++i) ui.vueExifdata->setRowHeight(i,hauteurLigne);
  ui.vueExifdata->resizeColumnsToContents();
}

void DialAPNList::changeApn(const QModelIndex &index, const QModelIndex &)
{  //! affiche les données exif du nouvel apn sélectionné
  ui.btDelete->setEnabled(true);
  ui.btEdit->setEnabled(true);
  APN apn = apns.at(index.row());
  emodel.clear();
  QStringList entete;
  entete << tr("Exif") << tr("Value");
  emodel.setHorizontalHeaderLabels(entete);
  apn.exif().profil2Model(emodel);
  for(int i=0; i<emodel.rowCount(); ++i) ui.vueApndata->setRowHeight(i,hauteurLigne);
  ui.vueApndata->resizeColumnsToContents();
}

void DialAPNList::ajoute()
{
  Exifs ex = exs.at(ui.vueExifs->currentIndex().row());
  APN apn (ex);
  apn.setName(newapns.at(ui.vueExifs->currentIndex().row()).Name());
  apns.append(apn);
  QList<QStandardItem *> mots;
  mots.append(new QStandardItem(apn.Name()));
  mots.at(0)->setEditable(false);
  amodel.appendRow(mots);
  ui.vueApns->setRowHeight(apns.count()-1,hauteurLigne);
}

void DialAPNList::supprime()
{
  APN apn = apns.at(ui.vueApns->currentIndex().row());
  QSettings settings("MadB", "ExifRename");
  settings.remove(QString("APNs/%1").arg(apn.Name()));
  apns.removeAt(ui.vueApns->currentIndex().row());
  amodel.removeRow(ui.vueApns->currentIndex().row());
}

void DialAPNList::edite()
{
  EditAPN dial(this,apns[ui.vueApns->currentIndex().row()]);
  dial.show();
  if (dial.exec()) {
    dial.saveApn(apns[ui.vueApns->currentIndex().row()]);
    amodel.setData(ui.vueApns->currentIndex(),QVariant(apns[ui.vueApns->currentIndex().row()].Name()));
  }
}

//---------------------------
EditAPN::EditAPN(QWidget *parent, APN &apn) :
  QDialog(parent)
{
  ui.setupUi(this);
  setModal(true);
  ui.leNom->setText(apn.Name());
  ui.leSuffix->setText(apn.Suf());
  ui.rbKeeptime->setChecked(apn.Op()==0);
  ui.rbAddtime->setChecked(apn.Op()==1);
  ui.rbSubtime->setChecked(apn.Op()==2);
  ui.spDays->setValue(apn.Days());
  ui.timeEdit->setTime(apn.Time());
  setWindowTitle(apn.Name());
}

void EditAPN::saveApn(APN &apn)
{
  if (apn.Name()!=ui.leNom->text()) {
    QSettings settings("MadB", "ExifRename");
    settings.remove(QString("APNs/%1").arg(apn.Name()));
  }
  apn.setName(ui.leNom->text());
  apn.setSuf(ui.leSuffix->text());
  int op = 0;
  if (ui.rbAddtime->isChecked()) op = 1;
  if (ui.rbSubtime->isChecked()) op = 2;
  apn.setOp(op);
  apn.setDays(ui.spDays->value());
  apn.setTime(ui.timeEdit->time());
}

//---------------------------
MonImage::MonImage(QWidget *parent) : QFrame(parent), image()
{
}

void MonImage::paintEvent(QPaintEvent *e)
{
  QFrame::paintEvent(e);
  if (!image.isNull()) {
    QPainter painter(this);
    QRectF dest       = contentsRect();
    qreal echelle     = qMin((dest.width()/(qreal) image.width()), (dest.height()/(qreal) image.height()));
    dest.setWidth(image.width()*echelle);
    dest.setHeight(image.height()*echelle);
    dest.moveLeft((contentsRect().width()-dest.width())/2);
    dest.moveTop((contentsRect().height()-dest.height())/2);
    painter.drawImage(dest.toRect(),image.scaled(dest.size().toSize(),Qt::KeepAspectRatio));
  }
}

void MonImage::setFile(QString fic)
{
  if (fic.isEmpty())
    image = QImage();
  else
    image.load(fic);
}

//---------------------------
AboutDialog::AboutDialog(QWidget *parent) : QDialog(parent)
{
  ui.setupUi(this);
  connect(ui.btFermer,  SIGNAL(clicked()), this, SLOT(close()));
}

//---------------------------
Aide::Aide(QWidget *parent) : QDialog(parent)
{
  ui.setupUi(this);
  QFile file(":/new/prefix1/index.html");
  if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
      return;
  ui.textBrowser->setHtml(QString::fromUtf8(file.readAll()));
}
