#include <exiv2.hpp>
#include "fenetre.h"

int           hauteurLigne = 16;
QList<APN>    apns;

//---------------------------
Fenetre::Fenetre()
{
  ui.setupUi(this);

  filtrimages << "*.jpg" << "*.jpeg" << "*.png" << "*.tiff" << "*.avi" << "*.mov";
  ui.vueFic->setModel(&fmodel);
  ui.vueFic->verticalHeader()->hide();
  //----- données exif
  ui.vueExif->setModel(&emodel);
  ui.vueExif->setSortingEnabled(true);
  ui.vueExif->setSelectionBehavior(QAbstractItemView::SelectRows);
  ui.vueExif->verticalHeader()->hide();
  motifChange = false;

  readSettings();
  changeMotif();
  QObject::connect(ui.btIdentCams,SIGNAL(clicked()),                    this, SLOT(identifier()));
  QObject::connect(ui.btPreview,  SIGNAL(clicked()),                    this, SLOT(prepare()));
  QObject::connect(ui.btApply,    SIGNAL(clicked()),                    this, SLOT(appliquer()));
  QObject::connect(ui.btBrowse,   SIGNAL(clicked()),                    this, SLOT(choisirDir()));
  QObject::connect(ui.btReload,   SIGNAL(clicked()),                    this, SLOT(relireDir()));
  QObject::connect(ui.ckPreview,  SIGNAL(stateChanged(int)),            this, SLOT(tooglePreview(int)));
  QObject::connect(ui.ckRecurs,   SIGNAL(stateChanged(int)),            this, SLOT(relireDir()));
  QObject::connect(ui.ckMoveup,   SIGNAL(stateChanged(int)),            this, SLOT(relireDir()));
  QObject::connect(ui.ckPrefix,   SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.lePrefix,   SIGNAL(textChanged(const QString &)), this, SLOT(changeMotif()));
  QObject::connect(ui.ckDate,     SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.cbDate,     SIGNAL(currentIndexChanged(int)),     this, SLOT(changeMotif()));
  QObject::connect(ui.ckSep1,     SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.leSep1,     SIGNAL(textChanged(const QString &)), this, SLOT(changeMotif()));
  QObject::connect(ui.ckTime,     SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.cbTime,     SIGNAL(currentIndexChanged(int)),     this, SLOT(changeMotif()));
  QObject::connect(ui.ckSep2,     SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.leSep2,     SIGNAL(textChanged(const QString &)), this, SLOT(changeMotif()));
  QObject::connect(ui.ckOrig,     SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.cbCreatime, SIGNAL(currentIndexChanged(int)),     this, SLOT(changeMotif()));
  QObject::connect(ui.ckSufcam,   SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.ckSuftime,  SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.rbDefExif,  SIGNAL(clicked(bool)),                this, SLOT(changeMotif()));
  QObject::connect(ui.rbDefFile,  SIGNAL(clicked(bool)),                this, SLOT(changeMotif()));
  QObject::connect(ui.leSuffix,   SIGNAL(textChanged(const QString &)), this, SLOT(changeMotif()));
  QObject::connect(ui.rbKeeptime, SIGNAL(clicked(bool)),                this, SLOT(changeMotif()));
  QObject::connect(ui.rbAddtime,  SIGNAL(clicked(bool)),                this, SLOT(changeMotif()));
  QObject::connect(ui.rbSubtime,  SIGNAL(clicked(bool)),                this, SLOT(changeMotif()));
  QObject::connect(ui.spDays,     SIGNAL(valueChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.timeEdit,   SIGNAL(timeChanged(const QTime &)),   this, SLOT(changeMotif()));
  QObject::connect(ui.cbExt,      SIGNAL(currentIndexChanged(int)),     this, SLOT(changeMotif()));
  QObject::connect(ui.ckSep3,     SIGNAL(stateChanged(int)),            this, SLOT(changeMotif()));
  QObject::connect(ui.leSep3,     SIGNAL(textChanged(const QString &)), this, SLOT(changeMotif()));
  QObject::connect(ui.leAutoZero, SIGNAL(textChanged(const QString &)), this, SLOT(changeMotif()));
  QObject::connect(ui.vueApns,    SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(editeApn()));
  QObject::connect(ui.vueFic->selectionModel(),  SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)),
                   this,       SLOT(changeFic(const QModelIndex &, const QModelIndex &)));
  QObject::connect(ui.vueFic,  SIGNAL(doubleClicked (const QModelIndex &)),
                   this,       SLOT(voirExif(const QModelIndex &)));
  QObject::connect(ui.action_Undo, SIGNAL(triggered()), this, SLOT(annuler()));

  QObject::connect(ui.action_AboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
  QObject::connect(ui.action_About,   SIGNAL(triggered()), this, SLOT(about()));
  QObject::connect(ui.action_Help,   SIGNAL(triggered()), this, SLOT(aide()));

  ui.cbCreatime->setCurrentIndex(0);
  majApns();
  ui.pgPrepare->reset();
}

void Fenetre::closeEvent(QCloseEvent *event)
{
  writeSettings();
  event->accept();
}

void Fenetre::readSettings()
{
  QSettings settings("MadB", "ExifRename");
  QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
  QSize size = settings.value("size", QSize(400, 400)).toSize();
  resize(size);
  move(pos);
  ui.partageH->restoreState(settings.value("partageH").toByteArray());
//   ui.partageV->restoreState(settings.value("partageV").toByteArray());
  curRep = settings.value("curRep", QString(QDir::currentPath())).toString();
  settings.beginGroup("APNs");
  QStringList cles = settings.childGroups();
  for(int i=0; i<cles.size();++i)
    apns.append(APN(cles.at(i),settings));
  settings.endGroup();
  lireDir(curRep);
}

void Fenetre::writeSettings()
{
  QSettings settings("MadB", "ExifRename");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.setValue("partageH", ui.partageH->saveState());
//   settings.setValue("partageV", ui.partageV->saveState());
  settings.setValue("curRep", curRep);
  settings.beginGroup("APNs");
  for(int i=0; i<apns.size(); ++i)
    apns[i].save(settings);
  settings.endGroup();
}

void Fenetre::about()
{
  AboutDialog ab(this);
  ab.setVersion(QString(APP_VERSION));
  ab.setExifVersion(Exiv2::version());
  ab.show();
  ab.exec();
}

void Fenetre::aide()
{
  Aide aid(this);
  aid.show();
  aid.exec();
}

void Fenetre::choisirDir()
{
  QString directory = QFileDialog::getExistingDirectory(this,
                                 tr("Choose a folder containing images"),
                                 ui.lbDir->text(),
                                 QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly);
  if (!directory.isEmpty())
    lireDir(directory);
}

void Fenetre::appendDir(QString dir, bool recursif)
{
  QDir rep(curRep+"/"+dir);
  QFileInfoList ficlist;
  ficlist = rep.entryInfoList(filtrimages);
  for (int l=0; l<ficlist.size(); ++l)
    srcFics << dir+ficlist[l].fileName();
  if (recursif) {
    ficlist = rep.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);
    for (int l=0; l<ficlist.size(); ++l)
      appendDir(QString("%1%2/").arg(dir).arg(ficlist[l].fileName()),recursif);
  }
}

void Fenetre::changeFic(const QModelIndex &index, const QModelIndex &)
{
  ui.stImage->setCurrentIndex(0);
  ui.vueFic->setCurrentIndex(index);
  emodel.clear();
  ui.vueImage->setFile(QString("%1/%2").arg(curRep).arg(srcFics.at(index.row())));
  ui.vueImage->update();
}

void Fenetre::voirExif(const QModelIndex &index)
{
  if (ui.stImage->currentIndex()==0) {
    ui.stImage->setCurrentIndex(1);
    int cpt = allExifsToModel(QString("%1/%2").arg(curRep).arg(srcFics.at(index.row())), emodel);

    for(int i=0; i<cpt; ++i) ui.vueExif->setRowHeight(i,hauteurLigne);
    for(int i=0; i<emodel.columnCount(); ++i)
      if (i==4)
        ui.vueExif->setColumnWidth(i,150);
      else
        ui.vueExif->resizeColumnToContents(i);
  }
  else changeFic(index, index);
}

void Fenetre::lireDir(QString &dir)
{
  curRep = dir;
  ui.lbDir->setText(curRep);
  QApplication::setOverrideCursor(Qt::WaitCursor);
  fmodel.clear();
  srcFics.clear();
  dstFics.clear();
  appendDir(QString(),ui.ckRecurs->isChecked());   // ajoute dans srcFics
  ui.lbFiles->setText(tr("%1 files").arg(srcFics.size()));
  QApplication::restoreOverrideCursor();
  emodel.clear();
  ui.stImage->setCurrentIndex(0);
  prepare(false);
} // lireDir

void Fenetre::relireDir() {
  ui.ckMoveup->setEnabled(ui.ckRecurs->isChecked());
  lireDir(curRep);
}

void Fenetre::changeMotif()
{
  motifChange = true;
  QDateTime dec(QDate::currentDate(),QTime(0,0,0));
  if (ui.cbCreatime->currentIndex()==1) {
    if (ui.rbAddtime->isChecked()) { dec = dec.addSecs(QTime().secsTo(ui.timeEdit->time())); }
    if (ui.rbSubtime->isChecked()) { dec = dec.addSecs(-QTime().secsTo(ui.timeEdit->time())); }
  }
  QString motif("<b>");
  if (ui.ckPrefix->isChecked()) motif.append(ui.lePrefix->text());
  if (ui.ckDate->isChecked())   motif.append(QDate(dec.date()).toString(ui.cbDate->currentText()));
  if (ui.ckSep1->isChecked())   motif.append(ui.leSep1->text());
  if (ui.ckTime->isChecked())   motif.append(QTime(dec.time()).toString(ui.cbTime->currentText()));
  if (ui.ckSep2->isChecked())   motif.append(ui.leSep2->text());
  if (ui.ckOrig->isChecked())   motif.append("IMG_000");
//   QMessageBox::information(this, "Infos", QString("Choix n°%1").arg(ui.cbCreatime->currentIndex()));
  switch (ui.cbCreatime->currentIndex()) {
    case 0 :
      if (ui.ckSufcam->isChecked())   motif.append(tr("-DC"));
      break;
    case 1 :
      motif.append(ui.leSuffix->text());
      break;
  }
  QString ext(".Jpg");
  switch (ui.cbExt->currentIndex()) {
    case 0 :
      ext = ext.toLower();
      break;
    case 1 :
      ext = ext.toUpper();
      break;
  }
  motif.append(ext);
  motif.append("</b>");
  ui.lbExample->setText(motif);
  prepare(false);
} // changeMotif

QList<QStandardItem *> Fenetre::creeColonne(QStringList &list, QList<bool> &col, QBrush &colfile)
{
  QList<QStandardItem *> mots;
  for (int l=0; l<list.size(); ++l) {
    mots.append(new QStandardItem(list[l]));
    mots.at(l)->setEditable(false);
    if (!col.at(l)) mots.at(l)->setBackground(colfile);
  }
  return mots;
} // creeColonne

void Fenetre::prepare(bool force)
{
  QApplication::setOverrideCursor(Qt::WaitCursor);
  ui.lbFiles->setText(tr("%1 files").arg(srcFics.size()));
  QBrush coldef(QColor(255,255,255));
  QBrush colsrc(QColor(230,230,230));
  QBrush colcol(QColor(200,255,200));
  fmodel.clear();
  dstFics.clear();
  ui.pgPrepare->setRange(0,srcFics.size());      // initialise la barre de progression
  if (srcFics.size()==0) ui.pgPrepare->setRange(0,1);
  ui.btApply->setEnabled(false);

  if ((ui.ckPreview->isChecked()) || force)
  {
    QList<bool> srcDates;                        // flags sur la provenance des dates (false : exif, true : fichier)
    QStringList apnames;                         // colonne des APNs
    bool dtexif;
    QString apname;
    for (int i=0; i<srcFics.size(); ++i) {
      ui.pgPrepare->setValue(i);                 // fait avancer la barre de progression
      dstFics  << renomme(srcFics.at(i), dtexif, apname);
      srcDates << dtexif;
      apnames  << apname;
    }
    // Détection des collisions (noms identiques)
    QList<bool> nocollision;
    QList<int>  collindice;                                    // indice de collision
    for(int i=0; i<dstFics.size(); ++i) {
      nocollision << true;                                     // pas de collision par défaut
      collindice  << 0;
    }
    for(int i=0; i<dstFics.size()-1; ++i)
      if (nocollision.at(i)) {
        int tot = 1;
        for(int j=i+1; j<dstFics.size(); ++j)
          if (nocollision.at(j)) {
            if (dstFics.at(i)==dstFics.at(j)) {
              nocollision[i]=nocollision[j]=false;                 // ya une collision
              collindice[i] = 1;
              collindice[j] = ++tot;
            }
        }
      }
    // Modifications suite aux collisions
    QString sep;
    if (ui.ckSep3->isChecked()) sep = ui.leSep3->text();
    for(int i=0; i<dstFics.size(); ++i) {
      if (!nocollision.at(i)) {
        dstFics[i].insert(dstFics[i].lastIndexOf("."),
                          QString("%1%2").arg(sep).arg(collindice.at(i),ui.leAutoZero->text().toInt(),10,QLatin1Char('0')));
      }
    }

    fmodel.appendColumn(creeColonne(apnames, srcDates,coldef));
    fmodel.appendColumn(creeColonne(srcFics, srcDates,colsrc));
    fmodel.appendColumn(creeColonne(dstFics, nocollision,colcol));
    ui.btApply->setEnabled(true);
  }
  else {
    QList<bool> bools;
    QStringList vides;
    for(int i=0; i<srcFics.size(); ++i) {
      bools << true;
      vides << "...";
    }
    fmodel.appendColumn(creeColonne(vides, bools,coldef));
    fmodel.appendColumn(creeColonne(srcFics, bools,colsrc));
    fmodel.appendColumn(creeColonne(vides, bools,colcol));
  }
  setHeader();
  ui.pgPrepare->reset();
  ui.vueImage->setFile("");
  motifChange = false;
  ui.vueImage->update();
  QApplication::restoreOverrideCursor();
} // prepare

void Fenetre::appliquer()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);
  QString memRep = QDir::currentPath();
  QDir::setCurrent(curRep);
  QStringList errFics;
  ui.pgPrepare->setRange(0,srcFics.size());      // initialise la barre de progression
  for (int i=0; i<srcFics.size(); ++i) {
    QFile fic(srcFics.at(i));
    if (!fic.rename(dstFics.at(i))) errFics << srcFics.at(i);
    ui.pgPrepare->setValue(i);                   // fait avancer la barre de progression
  }
  QDir::setCurrent(memRep);
  // Empiler pour annulation
  t_undo annule;
  annule.rep     = curRep;
  annule.sources = srcFics;
  annule.dests   = dstFics;
  stackUndo.push(annule);
  ui.action_Undo->setEnabled(true);

  ui.pgPrepare->reset();
  QApplication::restoreOverrideCursor();
  // Message des fichiers en erreur
  if (!errFics.isEmpty())
    QMessageBox::information(this,"Nombre d'erreurs",QString("%1 erreurs").arg(errFics.size()));
  relireDir();
} // appliquer

void Fenetre::setHeader()
{
  QStringList entete;
  entete << tr("DC") << tr("Current name") << tr("New name");
  fmodel.setHorizontalHeaderLabels(entete);
  for(int i=0; i<srcFics.size(); ++i) ui.vueFic->setRowHeight(i,hauteurLigne);
  ui.vueFic->resizeColumnsToContents();
}

QString Fenetre::renomme(const QString &fic, bool &dateExif, QString &nom)
{
  QFileInfo inf(curRep+"/"+fic);
  Exifs eximage(curRep+"/"+fic);
  QString suprep = curRep;
  suprep.chop(1);
  QString rep = inf.absolutePath();
  rep.replace(suprep,QString(""));
  if (!rep.isEmpty()) rep.remove(0,1);

  dateExif = (ui.cbCreatime->currentIndex()==0) && ui.rbDefExif->isChecked() && (!eximage.exifDateOriginal().isNull());
  QDateTime dec;
  int apnum = -1;
  nom = "...";                                                       // appareil inconnu
  if (!eximage.isEmpty())                                            // si il ya a des exifs
    for(int i=0; i<apns.size(); ++i)                                 // trouver l'apn dans la liste
      if (apns.at(i).isEqual(eximage)) {
        apnum = i;
        nom = apns.at(apnum).Name();                                 // nom de l'appareil si trouvé
      }
  if (dateExif) {
    dec = eximage.exifDateOriginal();                                // initialisation avec la date des exifs
    if ((ui.ckSuftime->isChecked()) && (apnum!=-1)) {                // si appareil connu et demande de correction du temps
      if (apns.at(apnum).Op()==1) { dec = dec.addSecs((apns.at(apnum).Days()*86400)+QTime().secsTo(apns.at(apnum).Time())); }
      if (apns.at(apnum).Op()==2) { dec = dec.addSecs(-(apns.at(apnum).Days()*86400)-QTime().secsTo(apns.at(apnum).Time())); }
    }
  }
  else {
    dec = inf.lastModified();                                        // initialisation avec la date du fichier                                                     // soit la correction générale du fichier
    if (ui.rbAddtime->isChecked()) { dec = dec.addSecs((ui.spDays->value()*86400)+QTime().secsTo(ui.timeEdit->time())); }
    if (ui.rbSubtime->isChecked()) { dec = dec.addSecs(-(ui.spDays->value()*86400)-QTime().secsTo(ui.timeEdit->time())); }
  }
  QString motif;
  if (!ui.ckMoveup->isChecked()) {
    int pos = fic.lastIndexOf("/");
    if (pos!=-1) motif = fic.left(pos+1);
  }
  if (ui.ckPrefix->isChecked()) motif.append(ui.lePrefix->text());
  if (ui.ckDate->isChecked())   motif.append(QDate(dec.date()).toString(ui.cbDate->currentText()));
  if (ui.ckSep1->isChecked())   motif.append(ui.leSep1->text());
  if (ui.ckTime->isChecked())   motif.append(QTime(dec.time()).toString(ui.cbTime->currentText()));
  if (ui.ckSep2->isChecked())   motif.append(ui.leSep2->text());
  if (ui.ckOrig->isChecked())   motif.append(inf.baseName());
  if ((apnum==-1) || (ui.cbCreatime->currentIndex()==1))
    motif.append(ui.leSuffix->text());
  else if (ui.ckSufcam->isChecked()) motif.append(apns.at(apnum).Suf());
  QString ext(inf.completeSuffix());
  switch (ui.cbExt->currentIndex()) {
    case 0 :
      ext = ext.toLower();
      break;
    case 1 :
      ext = ext.toUpper();
      break;
  }
  motif.append(".").append(ext);
  return motif;
} // renomme

void Fenetre::identifier()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);
  DialAPNList dial(this,curRep,srcFics);
  dial.show();
  QApplication::restoreOverrideCursor();
  dial.exec();
  writeSettings();
  majApns();
  prepare(false);
}

void Fenetre::tooglePreview(int state)
{
  if (state==Qt::Checked) ui.btPreview->setEnabled(false);
  else                    ui.btPreview->setEnabled(true);
  relireDir();
}

void Fenetre::majApns()
{
  // liste des apns connus
  amodel.clear();
  for(int l=0; l<apns.size(); ++l) {
    QList<QStandardItem *> mots;
    mots.append(new QStandardItem(apns.at(l).Name()));
    mots.at(0)->setEditable(false);
    amodel.appendRow(mots);
  }
  ui.vueApns->setModel(&amodel);
  ui.vueApns->verticalHeader()->hide();
  ui.vueApns->horizontalHeader()->hide();
  for(int i=0; i<amodel.rowCount(); ++i) ui.vueApns->setRowHeight(i,hauteurLigne);
}

void Fenetre::editeApn()
{
  EditAPN dial(this,apns[ui.vueApns->currentIndex().row()]);
  dial.show();
  if (dial.exec()) {
    dial.saveApn(apns[ui.vueApns->currentIndex().row()]);
    amodel.setData(ui.vueApns->currentIndex(),QVariant(apns[ui.vueApns->currentIndex().row()].Name()));
  }
}

void Fenetre::annuler()
{
  QApplication::setOverrideCursor(Qt::WaitCursor);
  QString memRep = QDir::currentPath();
  t_undo restaure = stackUndo.pop();
  curRep = restaure.rep;
  srcFics = restaure.dests;
  dstFics = restaure.sources;
  QDir::setCurrent(curRep);
  QStringList errFics;
  ui.pgPrepare->setRange(0,srcFics.size());      // initialise la barre de progression
  for (int i=0; i<srcFics.size(); ++i) {
    QFile fic(srcFics.at(i));
    if (!fic.rename(dstFics.at(i))) errFics << srcFics.at(i);
    ui.pgPrepare->setValue(i);                   // fait avancer la barre de progression
  }
  QDir::setCurrent(memRep);
  lireDir(curRep);
  ui.pgPrepare->reset();
  QApplication::restoreOverrideCursor();

  if (stackUndo.isEmpty()) ui.action_Undo->setEnabled(false);
}
